/*
 iMUX, an intelligent multiplexer

 LOW Fuse Byte = 0xFF
*/

#include <avr/io.h>
#include <avr/sleep.h>
#include <avr/interrupt.h>

#define F_CPU 16000000L    	// Systemtakt in Hz
#define BAUD 2400L	 		// Baudrate

#include <util/delay.h>

// Berechnungen
#define UBRR_VAL ((F_CPU+BAUD*8)/(BAUD*16)-1)   // clever runden
#define BAUD_REAL (F_CPU/(16*(UBRR_VAL+1)))     // Reale Baudrate
#define BAUD_ERROR ((BAUD_REAL*1000)/BAUD) // Fehler in Promille+1000, 1000 = kein Fehler.
 
#if ((BAUD_ERROR<990) || (BAUD_ERROR>1010))
  #error Systematischer Fehler der Baudrate grsser 1% und damit zu hoch! 
#endif

int main(void) {
	
	// PORTs initialisieren
 
    DDRA = 0xFF;			// Ausgang
	PORTB = 0xFF;			// Pull-ups ein
	PORTC = 0xFF;			// Pull-ups ein
	PORTD = 0xFF;			// Pull-ups ein

	// TEST

	DDRB |= (1<<PB0);		// Test LED

	// Timer 0 initialisieren

	TIMSK |= (1<<TOIE0);	// Timer overflow interrupt enable
	TCCR0 = 4;				// Prescaler 256, Overflow Rate = F_CPU/65536

	// UART initialisieren

    UCSRB |= (1<<TXEN) | (1<RXEN) | (1<<RXCIE);	// UART RX und TX einschalten, RX Interrupt
    UCSRC |= (1<<URSEL)|(3<<UCSZ0);    // Asynchron 8N1 
 
    UBRRH = UBRR_VAL >> 8;
    UBRRL = UBRR_VAL & 0xFF;
 
	// Interrupts freigeben

	sei();

	// Endlose Hauptschleife, nicht viel zu tun :-)

	while(1) {
        set_sleep_mode(SLEEP_MODE_IDLE);
        sleep_mode();                   // in den Schlafmodus wechseln, Siebenschlfer ;-)
	}
}

// UART RX complete interrupt
 
ISR(USART_RXC_vect) {
    PORTA = UDR;            // Daten auslesen, dadurch wird auch der Interrupt gelscht
}

// Timer Interrupt

ISR(TIMER0_OVF_vect) {
	static uint8_t tmp;			// TEST
	tmp++;
	if (tmp==0) PORTB ^= (1<<PB0);	// toggle LED

	while (!(UCSRA & (1<<UDRE)));	// Warte auf freien Sendepuffer
	UDR = PINC;						// Daten losschicken

}
